# ------------------------------------------------------------------
# ESV
# ------------------------------------------------------------------
GETC = 			0x60
SETC = 			0x61
GET  = 			0x62
INFREQ =		0x63
SETGET = 		0x6E
SETRES =		0x71
GETRES =		0x72
INF =			0x73
INFC = 			0x74
INFC_RES =		0x7A
SETGET_RES =	0x7E
SETI_SNA = 		0x50
SETC_SND =		0x51
GET_SNA = 		0x52
INF_SNA = 		0x53
SETGET_SNA =	0x5E

MODES = {
	'auto':  	0x41,
	'cool':  	0x42,
	'heat':  	0x43,
	'dehumidify': 0x44,
	'fan_only':	0x45,
	'other': 	0x40
}

FAN_SPEED = {
	'auto':	        0x41,
	'minimum':  	0x31,
	'low':  		0x32,
	'medium-low': 	0x33,
	'medium':		0x34,
	'medium-high': 	0x35,
	'high':			0x36,
	'very-high':    0x37,
	'max':			0x38
}

ESV_CODES = {
	0x60: {'name': 'GetC', 'description': 'Property value write request (no response required)'},
	0x61: {'name': 'SetC', 'description': 'Property value write request (response required)'},
	0x62: {'name': 'Get', 'description': 'Property value read request'},
	0x63: {'name': 'INF_REQ', 'description': 'Property value notification request'},
	0x6E: {'name': 'SetGet', 'description': 'Property value write & read request'},
	0x71: {'name': 'Set_Res', 'description': 'Property value Property value write response'},
	0x72: {'name': 'Get_Res' , 'description': 'Property value read response'},
	0x73: {'name': 'INF' , 'description': 'Property value notification'},
	0x74: {'name': 'INFC', 'description': 'Property value notification (response required)'},
	0x7A: {'name': 'INFC_Res' , 'description': 'Property value notification response'},
	0x7E: {'name': 'SetGet_Res' , 'description': 'Property value write & read response'},
	0x50: {'name': 'SetI_SNA', 'description': 'Property value write request (response not possible)'},
	0x51: {'name': 'SetC_SNA' , 'description': 'Property value write request (response not possible)'},
	0x52: {'name': 'Get_SNA', 'description': 'Property value read (response not possible)'},
	0x53: {'name': 'INF_SNA', 'description': 'Property value notification (response not possible)'},
    0x5E: {'name': 'SetGet_SNA', 'description': 'Property value write & read (response not possible)'}
}



"""
mitsubishi_echonet is used to provide control to Mitsubishi (and possibly other)
HVAC WiFi controllers that use the ECHONET-LITE protocol
such as the Mitsubishi MAC-IF558 adaptor.

I originally planned for this to be a full blown ECHONET library, but at this
stage it will control the AC and thats it!

"""

import socket
import struct
import sys
import time
from .eojx import *
from .epc  import *
from .esv  import *
from .functions  import Function as F

ENL_PORT = 3610
ENL_MULTICAST_ADDRESS = "224.0.23.0"


"""
buildEchonetMsg is used to build an ECHONET-LITE control message in the form
of a byte string

param data: a dict representing the control message.
return: an int representing the control message.
"""
def buildEchonetMsg(data):
   try:
      # EHD is fixed to 0x1081 because I am lazy.
      message = 0x1081

      # validate TID (set a default value if none provided)
      # TODO - TID message overlap.
      if 'TID' not in data:
         data['TID'] = 0x01
      elif data['TID'] > 0xFFFF:
         raise ValueError('Transaction ID is larger then 2 bytes.')
      message = (message << 16) + data['TID']

      # append SEOJ
      message = (message << 24) + 0x05FF01

      # validate DEOJ
      if data['DEOJGC'] in EOJX_GROUP:
          message = (message << 8) + data['DEOJGC']
      else:
          raise ValueError('Value ' + str(hex(data['DEOJGC'])) + ' not a valid SEO Group code')

      if data['DEOJCC'] in EOJX_CLASS[data['DEOJGC']]:
          message = (message << 8) + data['DEOJCC']
      else:
          raise ValueError('Value ' + str(hex(data['DEOJCC'])) + ' not a valid SEO class code')
      message = (message << 8) + data['DEOJIC']

      # validate ESV by looking up the codes.
      if data['ESV'] in ESV_CODES:
          # ESV code is a string
          message = (message << 8) + data['ESV']
      else:
          raise ValueError('Value not in ESV code table')

      # validate OPC
      message = (message << 8) + len(data['OPC'])

      # You can have multiple OPC per transaction.
      for values in data['OPC']:
        # validate EPC
        message =  (message << 8) + values['EPC']
        if 'PDC' in values:
            message =  (message << 8) + values['PDC']
        # if PDC has a value then concat EDT to message
            if values['PDC'] > 0:
                message =  (message << 8 * values['PDC']) + values['EDT']
        else:
            message =  (message << 8) + 0x00
      return format(int(message), 'x')
# some sloppy error handling here.
   except ValueError as error:
        # print('Caught this error: ' + repr(error))
        quit()


"""
decodeEchonetMsg is used to build an ECHONET-LITE control message in the form
of a byte string

param bytes: A string representing the hexadecimal control message in ECHONET LITE
return: a dict representing the deconstructed ECHONET packet
"""
def decodeEchonetMsg(byte):
  data = {}
  try:
      data['EHD1'] = byte[0]
      if data['EHD1'] not in EHD1:
          raise ValueError('EHD1 Header invalid')
      data['EHD2'] = byte[1]
      if data['EHD2'] not in EHD2:
          raise ValueError('EHD2 Header invalid')
      data['TID'] = int.from_bytes(byte[2:4], byteorder='big')

      # Decode SEOJ
      data['SEOJGC'] = byte[4]
      data['SEOJCC'] = byte[5]
      data['SEOJIC'] = byte[6]

      # Decode DEOJ
      data['DEOJGC'] =  byte[7]
      data['DEOJCC'] =  byte[8]
      data['DEOJIC'] =  byte[9]

      # Decode Service property
      data['ESV'] =  byte[10]

      i = 0
      epc_pointer = 12
      data['OPC'] = []
      # decode multiple processing properties (OPC)
      while i <  (byte[11]):
          OPC = {}
          pdc_pointer = epc_pointer + 1
          edt_pointer = pdc_pointer + 1
          end_pointer = edt_pointer
          OPC['EPC'] = byte[epc_pointer]
          OPC['PDC'] =  byte[pdc_pointer]
          pdc_len = byte[pdc_pointer]
          end_pointer += pdc_len
          OPC['EDT'] = byte[edt_pointer:end_pointer]
          epc_pointer = end_pointer
          i += 1
          data['OPC'].append(OPC)

  except ValueError as error:
        print('Caught this error: ' + repr(error))
        quit()
  return data

"""
sendMessage is used for ECHONET Unicast and Multicast transcations
message is assumed a correctly formatted ECHONET string
sendMessage will receive multiple messages if multicast is used "eg 224.0.23.0"

param message: an int representing the ECHONET message
param ip_address: a string representing the IPv4 address e.g "1.2.3.4"

return: an array representing the received response from the ECHONET message
"""
def sendMessage(message, ip_address):
    data =[]
    transaction_group = (ip_address, ENL_PORT)
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    socket_open = True
    while True:
        try:
            sock.bind(('',ENL_PORT))
            break
        except OSError:
            # Wait for socket to be open:
            time.sleep(0.5)

    # Set a timeout so the socket does not block
    # indefinitely when trying to receive data.
    sock.settimeout(0.5)

    # Set the time-to-live for messages to 1 so they do not
    # go past the local network segment.
    ttl = struct.pack('b', 1)
    sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, ttl)

    try:
        sent = sock.sendto(bytearray.fromhex(message), transaction_group)
        # Look for responses from all recipients
        while True:
            try:
                payload, server = sock.recvfrom(1024)
            except socket.timeout:
                break
            else:
                # Received a packet.
                data.append({'server':server,'payload':payload})
    finally:
        sock.close()
    return data

"""
discover is used to identify ECHONET nodes. Original plan was for this library
to fully support a multitude of ECHONET devices.

return: an array of discovered ECHONET node objects.
"""
def discover(echonet_class = ""):
    eoa = []; # array containing echonet objects
    tx_payload = {
        'TID' : 0x01, # Transaction ID 1
        'DEOJGC': 0x0E,
        'DEOJCC': 0xF0,
        'DEOJIC': 0x00,
        'ESV' : 0x62,
        'OPC' : [{'EPC': 0xD6}]
    }
    # Build ECHONET discover messafge.
    message = buildEchonetMsg(tx_payload)

    # Send message to multicast group and receive data
    data = sendMessage(message, ENL_MULTICAST_ADDRESS);
    # Decipher received message for each node discovered:

    for node in data:
        rx = decodeEchonetMsg(node['payload'])
        if (tx_payload['DEOJGC'] == rx['SEOJGC'] and
        rx['TID'] == tx_payload['TID'] and
        rx['OPC'][0]['EPC'] == 0xd6):
            # Action EDT payload by calling applicable function using lookup table
            edt = EPC_CODE[rx['SEOJGC']][rx['SEOJCC']][rx['OPC'][0]['EPC']][1](rx['OPC'][0]['EDT'])
            e = eval(EPC_CODE[edt['eojgc']][edt['eojcc']]['class'])(node['server'][0], edt['eojci'])
            print('ECHONET lite node discovered at {} - {} class'.format(node['server'][0], EOJX_CLASS[edt['eojgc']][edt['eojcc']]))
            if echonet_class == EOJX_CLASS[edt['eojgc']][edt['eojcc']] or echonet_class == "":
                eoa.append(e)

    return eoa

"""
opCode is used to return details from a node using the lookup table

return: an dict containing the properties of the node.
"""
def getOpCode(ip_address, deojgc, deojcc, deojci, opc, tid=0x01):
        tx_payload = {
            'TID' : tid, # Transaction ID 1
            'DEOJGC': deojgc,
            'DEOJCC': deojcc,
            'DEOJIC': deojci,
            'ESV' : GET,
            'OPC' : opc
        }
        # Build ECHONET discover messafge.
        message = buildEchonetMsg(tx_payload)
        rx_data = sendMessage(message, ip_address)
        return_data = {}
        if len(rx_data) > 0:
            rx = decodeEchonetMsg(rx_data[0]['payload'])
            # Action EDT payload by calling applicable function using lookup table
            for value in rx['OPC']:
                rx_edt = value['EDT']
                rx_epc = value['EPC']
                if rx_epc in EPC_CODE[deojgc][deojcc]['functions']:
                    edt = EPC_CODE[deojgc][deojcc]['functions'][rx_epc][1](rx_edt)
                else:
                    edt = EPC_SUPER[rx_epc][1](rx_edt)
                return_data.update(edt)
        return return_data

def getAllPropertyMaps(ip_address, deojgc, deojcc, deojci):
    propertyMaps = {}
    property_map = getOpCode(ip_address, deojgc, deojcc, deojci, [{'EPC':0x9F},{'EPC':0x9E}])
    # setProperties = getOpCode(ip_address, deojgc, deojcc, deojci, [{'EPC':0x9E}])
    # print(property_map)
    for property in property_map:
        propertyMaps[property] = {}
        for value in property_map[property]:
            if value in EPC_CODE[0x01][0x30]['functions']:
                propertyMaps[property][EPC_CODE[0x01][0x30]['functions'][value][0]] = value
            elif value in EPC_SUPER:
                propertyMaps[property][EPC_SUPER[value][0]] = value
            # else:
                 #print("code not found: " + hex(value) )
    return propertyMaps

"""
Superclass for Echonet node objects.
"""
class EchoNetNode:

    """
    Construct a new 'EchoNet' object.

    :param instance: Instance ID
    :param netif: IP address of node
    """
    def __init__(self, instance = 0x1, netif="", polling = 10 ):
        self.netif = netif
        self.last_transaction_id = 0x1
        self.eojgc = None
        self.eojcc = None
        self.instance = instance
        self.available_functions = None
        self.status = False
        self.propertyMaps = {}

    """
    getMessage is used to fire ECHONET request messages to get Node information
    Assumes one EPC is sent per message.

    :param tx_epc: EPC byte code for the request.
    :return: the deconstructed payload for the response
    """
    def getMessage(self, epc, pdc = 0x00):
        self.last_transaction_id += 1
        opc = [{'EPC': epc, 'PDC': pdc}]
        edt = getOpCode(self.netif, self.eojgc, self.eojcc, self.instance, opc, self.last_transaction_id )
        return edt


    """
    setMessage is used to fire ECHONET request messages to set Node information
    Assumes one OPC is sent per message.

    :param tx_epc: EPC byte code for the request.
    :param tx_edt: EDT data relevant to the request.
    :return: the deconstructed payload for the response
    """
    def setMessage(self, tx_epc, tx_edt):
        self.last_transaction_id += 1
        tx_payload = {
        'TID' : self.last_transaction_id,
        'DEOJGC': self.eojgc ,
        'DEOJCC': self.eojcc ,
        'DEOJIC': self.instance,
        'ESV' : SETC,
        'OPC' : [{'EPC': tx_epc, 'PDC': 0x01, 'EDT': tx_edt}]
        }
        message = buildEchonetMsg(tx_payload)
        data = sendMessage(message, self.netif);
        rx = decodeEchonetMsg(data[0]['payload'])

        rx_epc = rx['OPC'][0]['EPC']
        rx_pdc = rx['OPC'][0]['PDC']
        if rx_epc == tx_epc and rx_pdc == 0x00:
            return True
        else:
            return False

    """
    getOperationalStatus returns the ON/OFF state of the node

    :return: status as a string.
    """
    def getOperationalStatus(self): # EPC 0x80
        return self.getMessage(0x80)


    """
    setOperationalStatus sets the ON/OFF state of the node

    :param status: True if On, False if Off.
    """
    def setOperationalStatus(self, status): # EPC 0x80
        return self.setMessage(0x80, 0x30 if status else 0x31)

    """
    On sets the node to ON.

    """
    def on (self): # EPC 0x80
        return self.setMessage(0x80, 0x30)

    """
    Off sets the node to OFF.

    """
    def off (self): # EPC 0x80
        return self.setMessage(0x80, 0x31)

    def fetchSetProperties (self): # EPC 0x80
        if 'setProperties' in self.propertyMaps:
            return self.propertyMaps['setProperties']
        else:
            return {}

    def fetchGetProperties (self): # EPC 0x80
        if 'getProperties' in self.propertyMaps:
            return self.propertyMaps['getProperties']
        else:
            return {}

"""Class for Home AirConditioner Objects"""
class HomeAirConditioner(EchoNetNode):

    """
    Construct a new 'HomeAirConditioner' object.
    In theory this would work for any ECHNONET enabled domestic AC.

    :param instance: Instance ID
    :param netif: IP address of node
    """
    def __init__(self, netif, instance = 0x1):
        EchoNetNode.__init__(self, instance, netif)
        self.eojgc = 0x01
        self.eojcc = 0x30
        self.available_functions = EPC_CODE[self.eojgc][self.eojcc]['functions']
        self.propertyMaps = getAllPropertyMaps(self.netif, self.eojgc, self.eojcc , self.instance)
        self.setTemperature = None
        self.roomTemperature = None
        self.mode = False
        self.fan_speed = None
        self.JSON = {}

    """
    update is used as a quick and dirty way of producing a dict useful for API polling etc
    This exists primarily for home assistant!

    return: A string with the following attributes:
    {'status': '###', 'set_temperature': ##, 'fan_speed': '###', 'room_temperature': ##, 'mode': '###'}

    """
    def update(self):
        # at this stage we only care about a subset of gettable attributes that are relevant
        # down the track i might try to pull all of them..
        attributes = [0x80, # Op status
                      0xB3, # Set temperature
                      0xA0, # fan speed
                      0xBB, # room temperature
                      0xB0] # mode
                      #0x8A] # manufactorers code
        opc = []
        self.last_transaction_id += 1
        for value in attributes:
          if value in self.propertyMaps['getProperties'].values():
             opc.append({'EPC': value})
        returned_data = getOpCode(self.netif, self.eojgc, self.eojcc, self.instance, opc, self.last_transaction_id )
        if returned_data is not False:
            self.JSON = returned_data
            self.setTemperature = self.JSON['set_temperature']
            self.mode = self.JSON['mode']
            self.fan_speed = self.JSON['fan_speed']
            self.roomTemperature = self.JSON['room_temperature']
            self.status = self.JSON['status']
        return returned_data

    """
    GetOperationaTemperature get the temperature that has been set in the HVAC

    return: A string representing the configured temperature.
    """
    def getOperationalTemperature(self):
        self.setTemperature = self.getMessage(0xB3)['set_temperature']
        return self.setTemperature


    """
    setOperationalTemperature get the temperature that has been set in the HVAC

    param temperature: A string representing the desired temperature.
    """
    def setOperationalTemperature(self, temperature):
        if self.setMessage(0xB3, int(temperature)):
            self.setTemperature = temperature
            return True
        else:
            return False

    """
    GetMode returns the current configured mode (e.g Heating, Cooling, Fan etc)

    return: A string representing the configured mode.
    """
    def getMode(self):
        self.mode = self.getMessage(0xB0)['mode']
        return self.mode

    """
    setMode set the desired mode (e.g Heating, Cooling, Fan etc)

    param mode: A string representing the desired mode.
    """
    def setMode(self, mode):
        if self.setMessage(0xB0, MODES[mode]):
            self.mode = mode
            return True
        else:
            return False
    """
    GetFanSpeed gets the current fan speed (e.g Low, Medium, High etc)

    return: A string representing the fan speed
    """
    def getFanSpeed(self):
        self.fan_speed = self.getMessage(0xA0)['fan_speed']
        return self.fan_speed


    """
    setFanSpeed set the desired fan speed (e.g Low, Medium, High etc)

    param fans_speed: A string representing the fan speed
    """
    def setFanSpeed(self, fan_speed):
        if self.setMessage(0xA0, FAN_SPEED[fan_speed]):
            self.fanspeed = fan_speed
            return True
        else:
            return False




#------------------------------------------------------------------
# EOJX1: Group Name
# ---------------------------------------------------------------- */
EOJX_GROUP = {
	0x00: 'Sensor-related device group',
	0x01: 'Air conditioner-related device group',
	0x02: 'Housing/facility-related device group',
	0x03: 'Cooking/housework-related device group',
	0x04: 'Health-related device group',
	0x05: 'Management/control-related device group',
	0x06: 'AV-related device group',
	0x0E: 'Profile group',
	0x0F: 'User definition group'
};

# ------------------------------------------------------------------
# EOJX2: Name
# ---------------------------------------------------------------- */
EOJX_CLASS = {
	0x00: { # Sensor-related device group
		0x01: 'Gas leak sensor',
		0x02: 'Crime prevention sensor',
		0x03: 'Emergency button',
		0x04: 'First-aid sensor',
		0x05: 'Earthquake sensor',
		0x06: 'Electric leak sensor',
		0x07: 'Human detection sensor',
		0x08: 'Visitor sensor',
		0x09: 'Call sensor',
		0x0A: 'Condensation sensor',
		0x0B: 'Air pollution sensor',
		0x0C: 'Oxygen sensor',
		0x0D: 'Illuminance sensor',
		0x0E: 'Sound sensor',
		0x0F: 'Mailing sensor',
		0x10: 'Weight sensor',
		0x11: 'Temperature sensor',
		0x12: 'Humidity sensor',
		0x13: 'Rain sensor',
		0x14: 'Water level sensor',
		0x15: 'Bath water level sensor',
		0x16: 'Bath heating status sensor',
		0x17: 'Water leak sensor',
		0x18: 'Water overflow sensor',
		0x19: 'Fire sensor',
		0x1A: 'Cigarette smoke sensor',
		0x1B: 'CO2 sensor',
		0x1C: 'Gas sensor',
		0x1D: 'VOC sensor',
		0x1E: 'Differential pressure sensor',
		0x1F: 'Air speed sensor',
		0x20: 'Odor sensor',
		0x21: 'Flame sensor',
		0x22: 'Electric energy sensor',
		0x23: 'Current value sensor',
		0x24: 'Daylight sensor',
		0x25: 'Water flow rate sensor',
		0x26: 'Micromotion sensor',
		0x27: 'Passage sensor',
		0x28: 'Bed presence sensor',
		0x29: 'Open/close sensor',
		0x2A: 'Activity amount sensor',
		0x2B: 'Human body location sensor',
		0x2C: 'Snow sensor',
		0x2D: 'Air pressure sensor'
	},
	0x01: { #/ Air conditioner-related device group
		0x30: 'Home air conditioner',
		0x31: 'Cold blaster',
		0x32: 'Electric fan',
		0x33: 'Ventilation fan',
		0x34: 'Air conditioner ventilation fan',
		0x35: 'Air cleaner',
		0x36: 'Cold blast fan',
		0x37: 'Circulator',
		0x38: 'Dehumidifier',
		0x39: 'Humidifier',
		0x3A: 'Ceiling fan',
		0x3B: 'Electric Kotatsu',
		0x3C: 'Electric heating pad',
		0x3D: 'Electric blanket',
		0x3E: 'Space heater',
		0x3F: 'Panel heater',
		0x40: 'Electric carpet',
		0x41: 'Floor heater',
		0x42: 'Electric heater',
		0x43: 'Fan heater',
		0x44: 'Battery charger',
		0x45: 'Package-type commercial air conditioner (indoor unit)',
		0x46: 'Package-type commercial air conditioner (outdoor unit)',
		0x47: 'Package-type commercial air conditioner thermal storage unit',
		0x48: 'Commercial fan coil unit',
		0x49: 'Commercial air conditioner cold source (chiller)',
		0x50: 'Commercial air conditioning hot source (boiler)',
		0x51: 'Air-conditioning VAV for commercial applications',
		0x52: 'Air handling unit (air-conditioning) for commercial applications',
		0x53: 'Unit-cooler',
		0x54: 'Condensing unit for commercial applications',
		0x55: 'Electric storage heater'
	},
	0x02: { # Housing/facility-related device group
		0x60: 'Electrically operated blind/shade',
		0x61: 'Electrically operated shutter',
		0x62: 'Electrically operated curtain',
		0x63: 'Electrically operated rain sliding door/shutter',
		0x64: 'Electrically operated gate',
		0x65: 'Electrically operated window',
		0x66: 'Automatically operated entrance door/sliding door',
		0x67: 'Garden sprinkler',
		0x68: 'Fire sprinkler',
		0x69: 'Fountain',
		0x6A: 'Instantaneous water heater',
		0x6B: 'Electric water heater',
		0x6C: 'Solar water heater',
		0x6D: 'Circulation pump',
		0x6E: 'Bidet-equipped toilet (with electrically warmed seat)',
		0x6F: 'Electric lock',
		0x70: 'Gas line valve',
		0x71: 'Home sauna',
		0x72: 'Hot water generator',
		0x73: 'Bathroom dryer',
		0x74: 'Home elevator',
		0x75: 'Electrically operated room divider',
		0x76: 'Horizontal transfer',
		0x77: 'Electrically operated clothes-drying pole',
		0x78: 'Septic tank',
		0x79: 'Home solar power generation',
		0x7A: 'Cold/hot water heat source equipment',
		0x7B: 'Floor heater',
		0x7C: 'Fuel cell',
		0x7D: 'Storage battery',
		0x7E: 'Electric vehicle charger/discharger',
		0x7F: 'Engine cogeneration',
		0x80: 'Electric energy meter',
		0x81: 'Water flow meter',
		0x82: 'Gas meter',
		0x83: 'LP gas meter',
		0x84: 'Clock',
		0x85: 'Automatic door',
		0x86: 'Commercial elevator',
		0x87: 'Distribution panel metering',
		0x88: 'Low voltage smart electric energy meter',
		0x89: 'Smart gas meter',
		0x8A: 'High voltage smart electric energy meter',
		0x90: 'General lighting',
		0x91: 'Single function lighting',
		0x99: 'Emergency lighting',
		0x9D: 'Equipment light',
		0xA0: 'Buzzer'
	},
	0x03: { # Cooking/housework-related device group
		0xB0: 'Coffee machine',
		0xB1: 'Coffee mill',
		0xB2: 'Electric hot water pot (Electric thermos)',
		0xB3: 'Electric stove',
		0xB4: 'Toaster',
		0xB5: 'Juicer, food mixer',
		0xB6: 'Food processor',
		0xB7: 'Refrigerator',
		0xB8: 'Combination microwave oven (Electronic oven)',
		0xB9: 'Cooking heater',
		0xBA: 'Oven',
		0xBB: 'Raice cooker',
		0xBC: 'Electronic jar',
		0xBD: 'Dish washer',
		0xBE: 'Dish dryer',
		0xBF: 'Electric rice card cooker',
		0xC0: 'Keep-warm machine',
		0xC1: 'Race mill',
		0xC2: 'Automatic bread cooker',
		0xC3: 'Slow cooker',
		0xC4: 'Electric pickles cooker',
		0xC5: 'Washing machine',
		0xC6: 'Clothes dryer',
		0xC7: 'Electric iron',
		0xC8: 'Trouser press',
		0xC9: 'Futon dryer',
		0xCA: 'Small article, shoes dryer',
		0xCB: 'Electric vacuum cleaner (including central vacuum cleaner)',
		0xCC: 'Disposer',
		0xCD: 'Electric mosquito catcher',
		0xCE: 'Commercial show case',
		0xCF: 'Commercial refrigerator',
		0xD0: 'Commercial hot case',
		0xD1: 'Commercial fryer',
		0xD2: 'Commercial microwave oven',
		0xD3: 'Washer and dryer',
		0xD4: 'Commercial show case outdoor unit'
	},
	0x04: { # Health-related device group
		0x01: 'Weighing machine',
		0x02: 'Clinical thermometer',
		0x03: 'Blood pressure meter',
		0x04: 'Blood suger meter',
		0x05: 'Body fat meter',
	},
	0x05: { # Management/control-related device group
		0xFC: 'Secure communication shared key setup node',
		0xFD: 'Switch (supporting JEM-A/HA terminals)',
		0xFE: 'Portable (mobile) terminal',
		0xFF: 'Controller'
	},
	0x06: { # AV-related device group
		0x01: 'Display',
		0x02: 'Television',
		0x03: 'Audio',
		0x04: 'Network camera'
	},
	0x0E: { # Profile group
		0xF0: 'Node profile'
	}
}



from .functions import Function as f
# ------------------------------------------------------------------
# EHD1: ECHONET Lite Header 1
# ---------------------------------------------------------------- */
EHD1 = {
 0x00: 'Not available',
 0x10: 'Conventional ECHONET Lite Specification'
}

# ------------------------------------------------------------------
# EHD1: ECHONET Lite Header 2
# ---------------------------------------------------------------- */
EHD2 = {
	0x81: 'Format 1 (specified message format)',
	0x82: 'Format 2 (arbitrary message format)'
}

# ------------------------------------------------------------------
# EPC set
# ---------------------------------------------------------------- */

EPC_CODE = {
	0x00: { # Sensor-related device class group
		0x01: { # Gas leak sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Gas leak occurrence status',
			0xBF: 'Gas leak occurrence status resetting'
		},
		0x02: { # Crime prevention sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Invasion occurrence status',
			0xBF: 'Invasion occurrence status resetting'
		},
		0x03: { # Emergency button class
			0x80: 'Operation status',
			0xB1: 'Emergency occurrence status',
			0xBF: 'Emergency occurrence status resetting'
		},
		0x04: { # First-aid sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'First-aid occurrence status',
			0xBF: 'First-aid occurrence status resetting'
		},
		0x05: { # Earthquake sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Earthquake occurrence status',
			0xBF: 'Earthquake occurrence status resetting'
		},
		0x06: { # Electric leak sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Electric leak occurrence status',
			0xBF: 'Electric leak occurrence status resetting'
		},
		0x07: { # Human detection sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Human detection status'
		},
		0x08: { # Visitor sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Visitor detection status',
			0xBE: 'Visitor detection holding time'
		},
		0x09: { # Call sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Call status',
			0xBE: 'Call holding time'
		},
		0x0A: { # Condensation sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Condensation detection status'
		},
		0x0B: { # Air pollution sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Air pollution detection status'
		},
		0x0C: { # Oxygen sensor class
			0x80: 'Operation status',
			0xE0: 'Measured value of oxygen concentration'
		},
		0x0D: { # Illuminance sensor class
			0x80: 'Operation status',
			0xE0: 'Measured illuminance value 1',
			0xE1: 'Measured illuminance value 2'
		},
		0x0E: { # Sound sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Sound detection status',
			0xBE: 'Sound detection holding time'
		},
		0x0F: { # Mailing sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Mailing detection status'
		},
		0x10: { # Weight sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Weight detection status'
		},
		0x11: { # Temperature sensor class
			0x80: 'Operation status',
			0xE0: 'Measured temperature value'
		},
		0x12: { # Humidity sensor class
			0x80: 'Operation status',
			0xE0: 'Measured value of relative humidity'
		},
		0x13: { # Rain sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Rain detection status'
		},
		0x14: { # Water level sensor class
			0x80: 'Operation status',
			0xB0: 'Water level over detection threshold level',
			0xB1: 'Water level over detection status',
			0xE0: 'Measured value of water level'
		},
		0x15: { # Bath water level sensor class
			0x80: 'Operation status',
			0xB0: 'Bath water level over detection threshold level',
			0xB1: 'Bath water level over detection status',
			0xE0: 'Measured value of bath water level'
		},
		0x16: { # Bath heating status sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Bath heating detection status'
		},
		0x17: { # Water leak sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Water leak detection status'
		},
		0x18: { # Water overflow sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Water overflow detection status'
		},
		0x19: { # Fire sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Fire occurrence detection status',
			0xBF: 'Fire occurrence detection status resetting'
		},
		0x1A: { # Cigarette smoke sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Smoke (cigarette) detection status'
		},
		0x1B: { # CO2 sensor class
			0x80: 'Operation status',
			0xE0: 'Measured value of CO2 concentration'
		},
		0x1C: { # Gas sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Gas detection status',
			0xE0: 'Measured value of gas concentration'
		},
		0x1D: { # VOC sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'VOC detection status',
			0xE0: 'Measured value of VOC concentration'
		},
		0x1E: { # Differential pressure sensor class
			0x80: 'Operation status',
			0xE0: 'Measured value of differential pressure'
		},
		0x1F: { # Air speed sensor class
			0x80: 'Operation status',
			0xE0: 'Measured value of air speed',
			0xE1: 'Air flow direction'
		},
		0x20: { # Odor sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Odor detection status',
			0xE0: 'Measured odor value'
		},
		0x21: { # Flame sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Flame detection status',
			0xBF: 'Flame detection status resetting'
		},
		0x22: { # Electric energy sensor class
			0x80: 'Operation status',
			0xE0: 'Cumulative amounts of electric energy',
			0xE1: 'Medium-capacity sensor instantaneous electric energy',
			0xE2: 'Small-capacity sensor instantaneous electric energy',
			0xE3: 'Large-capacity sensor instantaneous electric energy',
			0xE4: 'Cumulative amounts of electric energy measurement log',
			0xE5: 'Effective voltage value'
		},
		0x23: { # Current value sensor class
			0x80: 'Operation status',
			0xE0: 'Measured current value 1',
			0xE1: 'Rated voltage to be measured',
			0xE2: 'Measured current  value 2'
		},
		0x25: { # Water flow rate sensor class
			0x80: 'Operation status',
			0xE0: 'Cumulative flow rate',
			0xE2: 'Flow rate'
		},
		0x26: { # Micromotion sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Micromotion detection status',
			0xB2: 'Detection counter',
			0xBC: 'Sampling count',
			0xBD: 'Sampling cycle'
		},
		0x27: { # Passage sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xBE: 'Passage detection hold time',
			0xE0: 'Passage detection direction'
		},
		0x28: { # Bed presence sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Bed presence detection status'
		},
		0x29: { # Open/close sensor class
			0x80: 'Operation status',
			0xE0: 'Degree-of-opening detection status 1',
			0xB0: 'Detection threshold level',
			0xB1: 'Degree-of-opening detection status 2'
		},
		0x2A: { # Activity amount sensor class
			0x80: 'Operation status',
			0xE0: 'Activity amount level 1',
			0xE1: 'Maximum number of human body ID’s',
			0xE2: 'Activity amount level 2',
			0xE3: 'Human body existence information'
		},
		0x2B: { # Human body location sensor
			0x80: 'Operation status',
			0xE0: 'Human body detection location 1',
			0xE1: 'Maximum number of human body ID’s',
			0xE2: 'Human body detection location 2',
			0xE3: 'Human body existence information'
		},
		0x2C: { # Snow sensor class
			0x80: 'Operation status',
			0xB0: 'Detection threshold level',
			0xB1: 'Snow detection status'
		},
		0x2D: { # Air pressure sensor class
			0x80: 'Operation status',
			0xE0: 'Air pressure measurement'
		}
	},
	0x01: { # Air conditioner-related device class group
		0x30: { # Home air conditioner class
			"class": 'HomeAirConditioner',
			"functions": {
				0x80: ('Operation status', f._013080),
				0x8F: ('Operation power-saving', None),
				0xB0: ('Operation mode setting', f._0130B0 ),
				0xB1: ('Automatic temperature control setting', None),
				0xB2: ('Normal/high-speed/silent operation setting', None),
				0xB3: ('Set temperature value', f._0130B3),
				0xB4: ('Set value of relative humidity in dehumidifying mode', None),
				0xB5: ('Set temperature value in cooling mode', None),
				0xB6: ('Set temperature value in heating mode', None),
				0xB7: ('Set temperature value in dehumidifying mode', None),
				0xB8: ('Rated power consumption', None),
				0xB9: ('Measured value of current consumption', None),
				0xBA: ('Measured value of room relative humidity', None),
				0xBB: ('Measured value of room temperature', f._0130BB),
				0xBC: ('Set temperature value of user remote control', None),
				0xBD: ('Measured cooled air temperature', None),
				0xBE: ('Measured outdoor air temperature', None),
				0xBF: ('Relative temperature setting', None),
				0xA0: ('Air flow rate setting', f._0130A0),
				0xA1: ('Automatic control of air flow direction setting', None),
				0xA3: ('Automatic swing of air flow setting', None),
				0xA4: ('Air flow direction (vertical) setting', None),
				0xA5: ('Air flow direction (horizontal) setting', None),
				0xAA: ('Special state', f._0130AA),
				0xAB: ('Non-priority state', None),
				0xC0: ('Ventilation function setting', None),
				0xC1: ('Humidifier function setting', None),
				0xC2: ('Ventilation air flow rate setting', None),
				0xC4: ('Degree of humidification setting', None),
				0xC6: ('Mounted air cleaning method', None),
				0xC7: ('Air purifier function setting', None),
				0xC8: ('Mounted air refresh method', None),
				0xC9: ('Air refresher function setting', None),
				0xCA: ('Mounted self-cleaning method', None),
				0xCB: ('Self-cleaning function setting', None),
				0xCC: ('Special function setting', None),
				0xCD: ('Operation status of components', None),
				0xCE: ('Thermostat setting override function', None),
				0xCF: ('Air purification mode setting', None),
				0x90: ('ON timer-based reservation setting', None),
				0x91: ('ON timer setting (time)', None),
				0x92: ('ON timer setting (relative time)', None),
				0x94: ('OFF timer-based reservation setting', None),
				0x95: ('OFF timer setting (time)', None),
				0x96: ('OFF timer setting (relative time)', None)
			}
		},
		0x33: { # Ventilation fan class
			0x80: 'Operation status',
			0xBF: 'Ventilation auto setting',
			0xA0: 'Set value of ventilation air flow rate'
		},
		0x34: { # Air conditioner ventilation fan class
			0x80: 'Operation status',
			0xB4: 'Set value of room relative humidity',
			0xBF: 'Ventilation auto setting',
			0xBA: 'Measured value of room relative humidity',
			0xA0: 'Set value of ventilation air flow rate',
			0xE0: 'Heat exchanger operation setting',
			0xC0: 'Measured value of CO2 concentration',
			0xC1: 'Smoke (cigarette) detection status',
			0xC2: 'Pollution detection status'
		},
		0x35: { # Air cleaner class
			0x80: 'Operation status',
			0xE1: 'Filter change notice',
			0xA0: 'Air flow rate setting',
			0xC1: 'Smoke (cigarette) detection status',
			0xC2: 'Optical catalyst operation setting',
			0xC0: 'Air pollution detection status'
		},
		0x39: { # Humidifier class
			0x80: 'Operation status',
			0xC0: 'Humidifying setting 1',
			0xC1: 'Humidifying setting 2',
			0xB4: 'Measured value of relative humidity',
			0x94: 'Reservation set of OFF timer',
			0x96: 'Relative time value set of OFF timer',
			0xC2: 'Ion emission setting',
			0xC3: 'Implemented ion emission method',
			0xC4: 'Special operation mode setting',
			0xC5: 'Water amount level'
		},
		0x42: { # Electric heater class
			0x80: 'Operation status',
			0xB1: 'Automatic temperature control setting',
			0xB3: 'Temperature setting',
			0xBB: 'Measured room temperature',
			0xBC: 'Remotely set temperature',
			0xA0: 'Air flow rate setting',
			0x90: 'ON timer-based reservation setting',
			0x91: 'ON timer setting (time)',
			0x92: 'ON timer setting (relative time)',
			0x94: 'OFF timer-based reservation setting',
			0x95: 'OFF timer setting (time)',
			0x96: 'Set value of OFF timer relative time'
		},
		0x43: { # Fan heater class
			0x80: 'Operation status',
			0xB3: 'Temperature setting value',
			0xBB: 'Measured temperature',
			0xB1: 'Automatic temperature control setting',
			0x90: 'ON timer reservation setting',
			0x91: 'ON timer setting value (time)',
			0x92: 'ON timer setting value (relative time)',
			0x94: 'OFF timer reservation setting',
			0x95: 'OFF timer setting value (time)',
			0x96: 'OFF timer value ( relative time)',
			0xC0: 'Extensional operation setting',
			0xC1: 'Extensional operation timer time setting value',
			0xC2: 'Ion emission setting',
			0xC3: 'Implemented ion emission method',
			0xC4: 'Oil amount level'
		},
		0x45: { # Package-type commercial air conditioner (indoor unit) class
			0x80: 'Operation status',
			0xB0: 'Operation mode setting',
			0xB3: 'Temperature setting 1',
			0xB4: '“Relative humidity setting for ‘dehumidifica-tion’ mode” 1',
			0xB5: '“Temperature setting for ‘cooling’ mode” 1',
			0xB6: '“Temperature setting for ‘heating’ mode” 1',
			0xB7: '“Temperature setting for ‘dehumidifica-tion’ mode” 1',
			0xB8: 'Rated power consumption of indoor unit',
			0xB9: 'Measured electric current consumption of indoor unit',
			0xBA: 'Measured indoor relative humidity 1',
			0xBB: 'Measured indoor temperature 1',
			0xBF: 'Relative temperature setting',
			0xA0: 'Air flow rate setting',
			0xA4: '“Air flow direction (vertical)” setting',
			0xA5: '“Air flow direction (horizontal)” setting',
			0xAA: '“Special” state',
			0xAC: 'Thermostat state',
			0xAE: 'Current function (“automatic” operation mode)',
			0xC0: 'Ventilation mode setting',
			0xC1: 'Combined operation of indoor unit and total heat exchanger',
			0xC2: 'Ventilation air flow rate setting',
			0xCD: '“Disabling of air conditioner” setting',
			0xCA: 'Group information',
			0xCE: 'Thermostat setting override function',
			0xCF: 'Filter cleaning reminder lamp setting',
			0xDB: 'Measured power consumption of indoor unit',
			0xDC: 'Aperture of expansion valve',
			0xE3: 'Temperature setting 2',
			0xE4: '“Relative humidity setting for ‘dehumidification’ mode” 2',
			0xE5: '“Temperature setting for ‘cooling’ mode” 2',
			0xE6: '“Temperature setting for ‘heating’ mode” 2',
			0xE7: '“Temperature setting for ‘dehumidification’ mode” 2',
			0xEA: 'Measured indoor relative humidity 2',
			0xEB: 'Measured indoor temperature 2',
			0x90: '“ON timer-based reservation” setting',
			0x91: 'ON timer setting (time)',
			0x92: 'ON timer setting (relative time)',
			0x94: '“OFF timer-based reservation” setting',
			0x95: 'OFF timer setting (time)',
			0x96: 'OFF timer setting (relative time)'
		},
		0x46: { # Package-type commercial air conditioner (outdoor unit) class
			0x80: 'Operation status',
			0xB0: 'Operation mode setting',
			0xB8: 'Rated power consumption of outdoor unit',
			0xB9: 'Measured electric current consumption of outdoor unit',
			0xBE: 'Measured outdoor air temperature 1',
			0xAA: '“Special” state',
			0xCA: 'Group information',
			0xD0: 'Operation status of compressor',
			0xD1: 'Operation mode information',
			0xD2: 'Fan rotation speed',
			0xDB: 'Measured power consumption of outdoor unit',
			0xEE: 'Measured outdoor air temperature 2'
		},
		0x55: { # Electric storage heater class
			0x80: 'Operation status',
			0xB3: 'Temperature setting',
			0xB8: 'Rated power consumption',
			0xBB: 'Measured indoor temperature',
			0xBE: 'Measured outdoor temperature',
			0xA0: 'Air flow rate setting',
			0xA1: 'Fan operation status',
			0xC0: 'Heat storage operation status',
			0xC1: 'Heat storage temperature setting',
			0xC2: 'Measured stored heat temperature',
			0xC3: 'Daytime heat storage setting',
			0xC4: 'Daytime heat storage ability',
			0xC5: 'Midnight power duration setting',
			0xC6: 'Midnight power start time setting',
			0xC7: 'Radiation method',
			0xC8: 'Child lock setting',
			0xD0: 'Fan timer 1 setting',
			0xD1: 'Fan timer 1 ON time setting',
			0xD2: 'Fan timer 1 OFF time setting',
			0xD3: 'Fan timer 2 setting',
			0xD4: 'Fan timer 2 ON time setting',
			0xD5: 'Fan timer 2 OFF time setting'
		}
	},
	0x02: { # Housing/facility-related device class group
		0x60: { # Electrically operated blind/shade class
			0x80: 'Operation status',
			0x89: 'Fault description (Recoverable faults)',
			0x90: 'Timer operation setting',
			0xC2: 'Wind detection status',
			0xC3: 'Sunlight detection status',
			0xD0: 'Opening (extension) speed setting',
			0xD1: 'Closing (retraction) speed setting',
			0xD2: 'Operation time',
			0xD4: 'Automatic operation setting',
			0xE0: 'Open/close (extension/retraction) setting',
			0xE1: 'Degree-of-opening level',
			0xE2: 'Shade angle setting ',
			0xE3: 'Open/close (extension/retraction) speed',
			0xE5: 'Electric lock setting',
			0xE8: 'Remote operation setting status',
			0xE9: 'Selective opening (extension) operation setting',
			0xEA: 'Open/closed (extended/retracted) status',
			0xEE: 'One-time opening (extension) speed setting',
			0xEF: 'One-time closing (retraction) speed setting'
		},
		0x61: { # Electrically operated shutter class
			0x80: 'Operation status',
			0x89: 'Fault description (Recoverable faults)',
			0x90: 'Timer operation setting',
			0xD0: 'Opening speed setting',
			0xD1: 'Closing speed setting',
			0xD2: 'Operation time',
			0xE0: 'Open/close operation setting',
			0xE1: 'Degree-of-opening setting',
			0xE2: 'Blind angle setting',
			0xE3: 'Open/close speed setting',
			0xE5: 'Electric lock setting',
			0xE8: 'Remote operation setting status',
			0xE9: 'Selective degree-of-opening setting',
			0xEA: 'Open/closed status',
			0xED: 'Slit degree-of-opening setting',
			0xEE: 'One-time opening speed setting',
			0xEF: 'One-time closing speed setting'
		},
		0x63: { # Electrically operated rain sliding door/shutter class
			0x80: 'Operation status',
			0x89: 'Fault description (Recoverable faults)',
			0x90: 'Timer operation setting',
			0xD0: 'Opening speed setting',
			0xD1: 'Closing speed setting',
			0xD2: 'Operation time',
			0xE0: 'Open/close operation setting',
			0xE1: 'Degree-of-opening setting',
			0xE2: 'Blind angle setting',
			0xE3: 'Opening/closing',
			0xE5: 'Electric lock setting',
			0xE8: 'Remote operation setting status',
			0xE9: 'Selective degree-of-opening setting',
			0xEA: 'Open/closed status',
			0xED: 'Slit degree-of-opening',
			0xEE: 'One-time opening speed setting',
			0xEF: 'One-time closing speed setting'
		},
		0x64: { # Electrically operated gate class
			0x80: 'Operation status',
			0x89: 'Fault description (Recoverable faults)',
			0xD0: 'Opening speed setting',
			0xD1: 'Closing speed setting',
			0xD2: 'Operation time setting value',
			0xE0: 'Opening/closing operation setting',
			0xE1: 'Degree-of-opening setting',
			0xE3: 'Opening/closing speed setting',
			0xE5: 'Electric lock setting',
			0xE8: 'Remote operation',
			0xE9: 'Selective degree-of-opening setting',
			0xEA: 'Open/closed status',
			0xEE: 'One-time opening speed setting',
			0xEF: 'One-time closing speed setting'
		},
		0x65: { # Electrically operated window class
			0x80: 'Operation status',
			0x89: 'Fault description (Recoverable faults)',
			0x90: 'Timer operation setting',
			0xC0: 'Registered temperature detection status',
			0xC1: 'Rain detection status',
			0xD0: 'Opening speed setting',
			0xD1: 'Closing speed setting',
			0xD2: 'Operation time setting value',
			0xD4: 'Automatic operation setting',
			0xE0: 'Opening/closing operation setting',
			0xE1: 'Degree-of-opening setting',
			0xE3: 'Opening/closing speed setting',
			0xE5: 'Electric lock setting',
			0xE8: 'Remote operation setting status',
			0xE9: 'Selective degree-of-opening setting',
			0xEA: 'Open/closed status',
			0xEE: 'One-time opening speed setting',
			0xEF: 'One-time closing speed setting'
		},
		0x66: { # Automatically operated entrance door/sliding door class
			0x80: 'Operation status',
			0x89: 'Fault description (Recoverable faults)',
			0xD0: 'Opening speed setting',
			0xD1: 'Closing speed setting',
			0xD2: 'Operation time setting value',
			0xD5: 'Opening time setting',
			0xE0: 'Opening/closing operation setting',
			0xE1: 'Degree-of-opening setting',
			0xE3: 'Opening/closing speed setting',
			0xE8: 'Remote operation setting status',
			0xE9: 'Selective degree-of-opening setting',
			0xEA: 'Open/closed status',
			0xEE: 'One-time opening speed setting',
			0xEF: 'One-time closing speed setting'
		},
		0x67: {  # Garden sprinkler class
			0x80: 'Operation status',
			0xE0: 'Sprinkle valve open/close setting',
			0xE1: 'Sprinkle interval setting',
			0xE2: 'Number of sprinkles setting',
			0xE3: 'Sprinkle time setting 1',
			0xE4: 'Sprinkle time setting 2',
			0xE5: 'Sprinkle duration setting'
		},
		0x6B: { # Electric water heater class
			0x80: 'Operation status',
			0xB0: 'Automatic water heating setting',
			0xB1: 'Automatic water temperature control setting',
			0xB2: 'Water heater status',
			0xB3: 'Water heating temperature setting',
			0xB4: 'Manual water heating stop days setting',
			0xB5: 'Relative time setting value for manual water heating OFF',
			0xB6: 'Tank operation mode setting',
			0xC0: 'Daytime reheating permission setting',
			0xC1: 'Measured temperature of water in water heater',
			0xC2: 'Alarm status',
			0xC3: 'Hot water supply status',
			0xC4: 'Relative time setting for keeping bath temperature',
			0xD1: 'Temperature of supplied water setting',
			0xD3: 'Bath water temperature setting',
			0xE0: 'Bath water volume setting',
			0xE1: 'Measured amount of water remaining in tank',
			0xE2: 'Tank capacity',
			0xE3: 'Automatic bath water heating mode setting',
			0xE9: 'Bathroom priority setting',
			0xEA: 'Bath operation status monitor',
			0xE4: 'Manual bath reheating operation setting',
			0xE5: 'Manual bath hot water addition function setting',
			0xE6: 'Manual slight bath water temperature lowering function setting',
			0xE7: 'Bath water volume setting 1',
			0xE8: 'Bath water volume setting 2',
			0xEE: 'Bath water volume setting 3',
			0xD4: 'Bath water volume setting 4',
			0xD5: 'Bath water volume setting 4-Maximum settable level',
			0x90: 'ON timer reservation setting',
			0x91: 'ON timer setting',
			0xD6: 'Volume setting',
			0xD7: 'Mute setting',
			0xD8: 'Remaining hot water volume',
			0xDB: 'Rated power consumption of H/P unit in wintertime',
			0xDC: 'Rated power consumption of H/P unit in in-between seasons',
			0xDD: 'Rated power consumption of H/P unit in summertime'
		},
		0x6E: { # Bidet-equipped toilet (with electrically warmed seat) class
			0x80: 'Operation status',
			0xE0: 'Temperature level of toilet seat',
			0xE1: 'Heater setting of toilet seat',
			0xE2: 'Temporal halt setting of toilet seat',
			0xE3: 'Temporal halt start time of toilet seat',
			0xE4: 'Temporal halt time duration of toilet seat',
			0xE5: 'Temperature level setting of room heating',
			0xE6: 'Room heating setting',
			0xE7: 'Room heating status',
			0xE8: 'Start time of room heating',
			0xE9: 'Duration time of room heating',
			0xEA: 'Special operation mode setting',
			0xEB: 'Human detection status',
			0xEC: 'Seating detection status'
		},
		0x6F: { # Electric lock class
			0x80: 'Operation status',
			0xE0: 'Lock setting 1',
			0xE1: 'Lock setting 2',
			0xE2: 'Lock status of door guard',
			0xE3: 'Door open/close status',
			0xE4: 'Occupant/non-occupant status',
			0xE5: 'Alarm status',
			0xE6: 'Auto lock mode setting',
			0xE7: 'Battery level'
		},
		0x72: { # Hot water generator class
			0x80: 'Operation status',
			0xD0: 'Hot water heating status',
			0xD1: 'Set value of hot water temperature',
			0xD2: 'Hot water warmer setting',
			0xDA: '“Duration of automatic operation” setting',
			0xDB: 'Remaining automatic operation time',
			0xE1: 'Set value of bath temperature',
			0xE2: 'Bath water heater status',
			0xE3: 'Bath auto mode setting',
			0xE4: 'Bath additional boil-up operation setting',
			0xE5: 'Bath hot water adding operation setting',
			0xE6: 'Bath water temperature lowering operation setting',
			0xE7: 'Bath hot water volume setting 1',
			0xE8: 'Bath hot water volume setting 2',
			0xEE: 'Bath hot water volume setting 3',
			0xD4: 'Bath hot water volume setting 4',
			0xD5: 'Bath hot water volume setting 4 - Maximum settable level',
			0xE9: 'Bathroom priority setting',
			0xEA: 'Shower hot water supply status',
			0xEB: 'Kitchen hot water supply status',
			0xEC: 'Hot water warmer ON timer reservation setting',
			0xED: 'Set value of hot water warmer ON timer time',
			0xEF: 'Bath operation status monitor',
			0x90: 'ON timer reservation setting',
			0x91: 'Set value of ON timer time',
			0x92: 'Set value of ON timer relative time',
			0xD6: 'Volume setting',
			0xD7: 'Mute setting'
		},
		0x73: { # Bathroom dryer class
			0x80: 'Operation status',
			0xB0: 'Operation setting',
			0xB1: 'Ventilation operation setting',
			0xB2: 'Bathroom pre-warmer operation setting',
			0xB3: 'Bathroom heater operation setting',
			0xB4: 'Bathroom dryer operation setting',
			0xB5: 'Cool air circulator operation setting',
			0xBA: 'Measured relative bathroom humidity',
			0xBB: 'Measured bathroom temperature',
			0xC2: 'Ventilation air flow rate setting',
			0xCF: 'Filter cleaning reminder sign setting',
			0xE0: 'Human body detection status',
			0x90: '“ON timer-based reservation” setting 1',
			0xE1: '“ON timer-based reservation” setting 2',
			0x91: 'ON timer setting (time)',
			0x92: 'ON timer setting (relative time)',
			0x94: '“OFF timer-based reservation” setting',
			0x95: 'OFF timer setting (time)',
			0x96: 'OFF timer setting (relative time)'
		},
		0x79: { # Home solar power generation class
			0x80: 'Operation status',
			0xD0: 'System-interconnected type',
			0xE0: 'Measured instantaneous amount of electricity generated',
			0xE1: 'Measured cumulative amount of electric energy generated',
			0xE2: 'Resetting cumulative amount of electric energy generated',
			0xE3: 'Measured cumulative amount of electric energy sold',
			0xE4: 'Resetting cumulative amount of electric energy sold',
			0xE5: 'Power generation output limit setting 1',
			0xE6: 'Power generation output limit setting 2',
			0xE7: 'Limit setting for the amount of electricity sold',
			0xE8: 'Rated power generation output (System-interconnected)',
			0xE9: 'Rated power generation output (Independent)'
		},
		0x7A: { # Cold/hot water heat source equipment class
			0x80: 'Operation status',
			0xE0: 'Operation mode setting',
			0xE1: 'Water temperature setting 1',
			0xE2: 'Water temperature setting 2',
			0xE3: 'Measured temperature of outward water (Exit water Temperature)',
			0xE4: 'Measured temperature of inward water (Entrance water temperature)',
			0xE5: 'Special operation setting',
			0xE6: 'Daily timer setting',
			0xE7: 'Daily timer setting 1',
			0xE8: 'Daily timer setting 2',
			0xE9: 'ON timer reservation setting',
			0x91: 'ON timer setting',
			0x92: 'Relative ON timer setting',
			0x94: 'OFF timer reservation setting',
			0x95: 'Time set by OFF timer',
			0x96: 'Relative OFF timer setting'
		},
		0x7B: { # Floor heater class
			0x80: 'Operation status',
			0xE0: 'Temperature setting 1',
			0xE1: 'Temperature setting 2',
			0xE2: 'Measured room temperature',
			0xE3: 'Measured floor temperature',
			0xE4: 'Zone change setting',
			0xE5: 'Special operation setting',
			0xE6: 'Daily timer setting',
			0xE7: 'Daily timer setting 1',
			0xE8: 'Daily timer setting 2',
			0x90: 'ON timer reservation setting',
			0x91: 'Time set by ON timer',
			0x92: 'Relative ON timer setting',
			0x94: 'OFF timer reservation setting',
			0x95: 'Time set by OFF timer',
			0x96: 'Relative OFF timer setting'
		},
		0x7C: { # Fuel cell class
			0x80: 'Operation status',
			0xC1: 'Measured temperature of water in water heater',
			0xC2: 'Rated power generation output',
			0xC3: 'Heating value of hot water storage tank',
			0xC4: 'Measured instantaneous power generation output',
			0xC5: 'Measured cumulative power generation output',
			0xC6: 'Cumulative power generation output reset setting',
			0xC7: 'Measured instantaneous gas consumption',
			0xC8: 'Measured cumulative gas consumption',
			0xC9: 'Cumulative gas consumption reset setting',
			0xCA: 'Power generation setting',
			0xCB: 'Power generation status',
			0xCC: 'Measured in-house instantaneous power consumption',
			0xCD: 'Measured in-house cumulative power consumption',
			0xCE: 'In-house cumulative power consumption reset',
			0xD0: 'System interconnected type',
			0xE1: 'Measured remaining hot water amount',
			0xE2: 'Tank capacity'
		},
		0x7D: { # Storage battery class
			0x80: 'Operation status',
			0xC8: 'Minimum/maxim um charging electric energy',
			0xC9: 'Minimum/maxim um discharging electric energy',
			0xCA: 'Minimum/maxim um charging current',
			0xCB: 'Minimum/maxim um discharging current',
			0xCF: 'Working operation status',
			0xD0: 'Rated electric energy',
			0xD1: 'Rated capacity',
			0xD2: 'Rated voltage',
			0xD3: 'Measured instantaneous charging/discharging electric energy',
			0xD4: 'Measured instantaneous charging/discharging current',
			0xD5: 'Measured instantaneous charging/discharging voltage',
			0xD6: 'Measured cumulative discharging electric energy',
			0xD7: '“Measured cumulative discharging electric energy” reset setting',
			0xD8: 'Measured cumulative charging electric energy',
			0xD9: '“Measured cumulative charging electric energy” reset setting',
			0xDA: 'Operation mode setting',
			0xDB: 'System-interconne cted type',
			0xDC: 'Minimum/maxim um charging power (Independent)',
			0xDD: 'Minimum/maxim um discharging power (Independent)',
			0xDE: 'Minimum/maxim um charging current (Independent)',
			0xDF: 'Minimum/maxim um discharging current (Independent)',
			0xE0: 'Charging/discharging amount setting 1',
			0xE1: 'Charging/discharging amount setting 2',
			0xE2: 'Remaining stored electricity 1',
			0xE3: 'Remaining stored electricity 2',
			0xE4: 'Remaining stored electricity 3',
			0xE5: 'Battery state of health',
			0xE6: 'Battery type',
			0xE7: 'Charging amount setting 1',
			0xE8: 'Discharging amount setting 1',
			0xE9: 'Charging amount setting 2',
			0xEA: 'Discharging amount setting 2',
			0xEB: 'Charging electric energy setting',
			0xEC: 'Discharging electric energy setting',
			0xED: 'Charging current setting',
			0xEE: 'Discharging current setting',
			0xEF: 'Rated voltage (Independent)'
		},
		0x7E: { # Electric vehicle charger/discharger class
			0x80: 'Operation status',
			0xC0: 'Dischargeable capacity of vehicle mounted battery 1',
			0xC1: 'Dischargeable capacity of  vehicle mounted battery 2',
			0xC2: 'Remaining dischargeable capacity of vehicle mounted battery 1',
			0xC3: 'Remaining dischargeable capacity of vehicle mounted battery 2',
			0xC4: 'Remaining dischargeable capacity of vehicle mounted battery 3',
			0xC5: 'Rated charge capacity',
			0xC6: 'Rated discharge capacity',
			0xC7: 'Vehicle connection and chargeable/dischargeable status',
			0xC8: 'Minimum/maximum charging electric energy',
			0xC9: 'Minimum/maximum discharging electric energy',
			0xCA: 'Minimum/maximum charging current',
			0xCB: 'Minimum/maximum discharging current',
			0xD0: 'Used capacity of vehicle mounted battery 1',
			0xD1: 'Used capacity of vehicle mounted battery 2',
			0xD2: 'Rated voltage',
			0xD3: 'Measured instantaneous charging/discharging electric energy',
			0xD4: 'Measured instantaneous charging/discharging current',
			0xD5: 'Measured instantaneous charging/discharging voltage',
			0xD6: 'Measured cumulative amount of discharging electric energy',
			0xD7: 'Cumulative amount of discharging electric energy reset setting',
			0xD8: 'Measured cumulative amount of charging electric energy',
			0xD9: 'Cumulative amount of charging electric energy reset setting',
			0xDA: 'Operation mode setting',
			0xDB: 'System-interconnecte d type',
			0xE2: 'Remaining stored electricity of vehicle mounted battery 1',
			0xE3: 'Remaining stored electricity of vehicle mounted battery 2',
			0xE4: 'Remaining stored electricity of vehicle mounted battery 3',
			0xE7: 'Charging amount setting 1',
			0xE9: 'Charging amount setting 2',
			0xEB: 'Charging electric energy setting',
			0xEC: 'Discharging electric energy setting',
			0xED: 'Charging current setting',
			0xEE: 'Discharging current setting',
			0xEF: 'Rated voltage (Independent)'
		},
		0x7F: { # Engine cogeneration class
			0x80: 'Operation status',
			0xC1: 'Measured temperature of water in water heater',
			0xC2: 'Rated power generation output',
			0xC3: 'Heating value of hot water storage tank',
			0xC4: 'Measured instantaneous power generation output',
			0xC5: 'Measured cumulative power generation output',
			0xC6: 'Cumulative power generation output reset setting',
			0xC7: 'Measured instantaneous gas consumption',
			0xC8: 'Measured cumulative gas consumption',
			0xC9: 'Cumulative gas consumption reset setting',
			0xCA: 'Power generation setting',
			0xCB: 'Power generation status',
			0xCC: 'Measured in-house instantaneous power consumption',
			0xCD: 'Measured in-house cumulative power consumption',
			0xCE: 'In-house cumulative power consumption reset',
			0xD0: 'System interconnected type',
			0xE1: 'Measured remaining hot water amount',
			0xE2: 'Tank capacity'
		},
		0x80: { # Electric energy meter class
			0x80: 'Operation status',
			0xE0: 'Cumulative amounts of electric energy measurement value',
			0xE2: 'Cumulative amounts of electric energy unit',
			0xE3: 'Cumulative amounts of electric energy measurement log 1',
			0xE4: 'Cumulative amounts of electric energy measurement log 2'
		},
		0x81: { # Water flow meter class
			0x80: 'Operation status',
			0xD0: 'Flowing water classification',
			0xD1: 'Owner classification',
			0xE0: 'Measured cumulative amount of flowing water',
			0xE1: 'Unit for measured cumulative amounts of flowing water',
			0xE2: 'Historical data of measured cumulative amounts of flowing water',
			0xE3: 'Detection of abnormal value in metering data',
			0xE4: 'Security data information',
			0xE5: 'ID number setting',
			0xE6: 'Verification expiration information'
		},
		0x82: { # Gas meter class
			0x80: 'Operation status',
			0xE0: 'Cumulative amount of gas consumption measurement value',
			0xE2: 'Cumulative amount of gas consumption measurement log'
		},
		0x83: { # LP gas meter class
			0x80: 'Operation status',
			0xE0: 'Cumulative amount of gas consumption of metering data 1',
			0xE1: 'Cumulative amount of gas consumption of metering data 2',
			0xE2: 'Error detection status of metering data',
			0xE3: 'Security data 1',
			0xE4: 'Security data 2',
			0xE5: 'Center valve shut-off status',
			0xE6: 'Center valve shut-off recovery permission setting status',
			0xE7: 'Emergency valve shut-off status',
			0xE8: 'Shut-off valve open/close status',
			0xE9: 'Residual volume control warning',
			0xEA: 'Set value of residual volume control warning level 1',
			0xEB: 'Set value of residual volume control warning level 2',
			0xEC: 'Set value of residual volume control warning level 3',
			0xED: 'Slight leak timer value (gas flow rate continuation)',
			0xEE: 'Slight leak timer value (without pressure increase)',
			0xEF: 'Shut-off reason log',
			0xD0: 'Maximum value of supply pressure data',
			0xD1: 'Minimum value of supply pressure data',
			0xD2: 'Current value of supply pressure data',
			0xD3: 'Maximum value of block pressure data',
			0xD4: 'Minimum value of block pressure data',
			0xD5: 'Current value of block pressure data',
			0xD6: 'Number of block pressure/supply pressure error days: times',
			0xD7: 'Test call setting'
		},
		0x87: { # Distribution panel metering class
			0x80: 'Operation status',
			0xC0: 'Measured cumulative amount of electric energy (normal direction)',
			0xC1: 'Measured cumulative amount of electric energy (reverse direction)',
			0xC2: 'Unit for cumulative amounts of electric energy',
			0xC3: 'Historical data of measured cumulative amounts of electric energy (normal direction)',
			0xC4: 'Historical data of measured cumulative amounts of electric energy (reverse direction)',
			0xC5: 'Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved',
			0xC6: 'Measured instantaneous amount of electric energy',
			0xC7: 'Measured instantaneous currents',
			0xC8: 'Measured instantaneous voltages',
			0xD0: 'Measurement channel 1',
			0xD1: 'Measurement channel 2',
			0xD2: 'Measurement channel 3',
			0xD3: 'Measurement channel 4',
			0xD4: 'Measurement channel 5',
			0xD5: 'Measurement channel 6',
			0xD6: 'Measurement channel 7',
			0xD7: 'Measurement channel 8',
			0xD8: 'Measurement channel 9',
			0xD9: 'Measurement channel 10',
			0xDA: 'Measurement channel 11',
			0xDB: 'Measurement channel 12',
			0xDC: 'Measurement channel 13',
			0xDD: 'Measurement channel 14',
			0xDE: 'Measurement channel 15',
			0xDF: 'Measurement channel 16',
			0xE0: 'Measurement channel 17',
			0xE1: 'Measurement channel 18',
			0xE2: 'Measurement channel 19',
			0xE3: 'Measurement channel 20',
			0xE4: 'Measurement channel 21',
			0xE5: 'Measurement channel 22',
			0xE6: 'Measurement channel 23',
			0xE7: 'Measurement channel 24',
			0xE8: 'Measurement channel 25',
			0xE9: 'Measurement channel 26',
			0xEA: 'Measurement channel 27',
			0xEB: 'Measurement channel 28',
			0xEC: 'Measurement channel 29',
			0xED: 'Measurement channel 30',
			0xEE: 'Measurement channel 31',
			0xEF: 'Measurement channel 32',
			0xB0: 'Master rated capacity',
			0xB1: 'Number of measurement channels (simplex)',
			0xB2: 'Channel range specification for cumulative amount of electric power consumption measurement (simplex)',
			0xB3: 'Measured cumulative amount of electric power consumption list (simplex)',
			0xB4: 'Channel range specification for instantaneous current measurement (simplex)',
			0xB5: 'Measured instantaneous current list (simplex)',
			0xB6: 'Channel range specification for instantaneous power consumption measurement (simplex)',
			0xB7: 'Measured instantaneous power consumption list (simplex)',
			0xB8: 'Number of measurement channels (duplex)',
			0xB9: 'Channel range specification for cumulative amount of electric power consumption measurement (duplex)',
			0xBA: 'Measured cumulative amount of electric power consumption list (duplex)',
			0xBB: 'Channel range specification for instantaneous current measurement (duplex)',
			0xBC: 'Measured instantaneous current list (duplex)',
			0xBD: 'Channel range specification for instantaneous power consumption measurement (duplex)',
			0xBE: 'Measured instantaneous power consumption list (duplex)'
		},
		0x88: { # Low voltage smart electric energy meter class
			0x80: 'Operation status',
			0xD3: 'Coefficient',
			0xD7: 'Number of effective digits for cumulative amounts of electric energy',
			0xE0: 'Measured cumulative amount of electric energy (normal direction)',
			0xE1: 'Unit for cumulative amounts of electric energy (normal and reverse directions)',
			0xE2: 'Historical data of measured cumulative amounts of electric energy 1 (normal direction)',
			0xE3: 'Measured cumulative amounts of electric energy (reverse direction)',
			0xE4: 'Historical data of measured cumulative amounts of electric energy 1 (reverse direction)',
			0xE5: 'Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved 1',
			0xE7: 'Measured instantaneous electric energy',
			0xE8: 'Measured instantaneous currents',
			0xEA: 'Cumulative amounts of electric energy measured at fixed time (normal direction)',
			0xEB: 'Cumulative amounts of electric energy measured at fixed time (reverse direction)',
			0xEC: 'Historical data of measured cumulative amounts of electric energy 2 (normal and reverse directions)',
			0xED: 'Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved 2'
		},
		0x89: { # Smart gas meter class
			0x80: 'Operation status',
			0xE0: 'Gas meter classification setting',
			0xE1: 'Owner classification setting',
			0xE2: 'Measured cumulative gas consumption',
			0xE3: 'Unit for measured cumulative gas consumption',
			0xE4: 'Historical data of measured cumulative gas consumption',
			0xE5: 'Day setting for which the historical data of measured cumulative gas consumption is to be retrieved',
			0xE6: 'Detection of abnormal value in metering data',
			0xE7: 'Security data information',
			0xE8: 'Valve closure by the Center',
			0xE9: 'Permission from the Center to reopen the valve closed by the Center',
			0xEA: 'Emergency closure of shutoff valve',
			0xEB: 'Shutoff valve status',
			0xEC: 'Historical data of shutoff reasons',
			0xED: 'ID number setting',
			0xEE: 'Verification expiration setting',
			0xD0: 'Measured cumulative gas consumption information with date and time',
			0xD1: 'Historical information of cumulative gas consumption'
		},
		0x8A: { # High voltage smart electric energy meter class
			0x80: 'Operation status',
			0xD3: 'Coefficient',
			0xD4: 'Multiplying factor for coefficient',
			0xE0: 'Fixed date',
			0xE1: 'Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved',
			0xE2: 'Measured cumulative amount of active electric energy',
			0xE3: 'Cumulative amounts of active electric energy at fixed time',
			0xE4: 'Measurement data of cumulative amount of active electric energy for power factor measurement',
			0xE5: 'Number of effective digits for cumulative amount of active electric energy',
			0xE6: 'Unit for cumulative amounts of active electric energy',
			0xE7: 'Historical data of measured cumulative amount of active electric energy',
			0xC1: 'Monthly maximum electric power demand',
			0xC2: 'Cumulative maximum electric power demand',
			0xC3: 'Electric power demand at fixed time (30-minute average electric power)',
			0xC4: 'Number of effective digits of electric power demand',
			0xC5: 'Unit of electric power demand',
			0xC6: 'Historical data of measured electric power demand',
			0xC7: 'Unit of cumulative maximum electric power demand',
			0xCA: 'Measurement data of reactive electric power consumption (lag) for power factor measurement',
			0xCB: 'Measurement data of cumulative amount of reactive electric power consumption (lag) at fixed time for power factor measurement',
			0xCC: 'Number of effective digits for measurement data of cumulative amount of reactive electric power consumption (lag) for power factor measurement',
			0xCD: 'Unit of measurement data of cumulative amount of reactive electric power consumption (lag)',
			0xCE: 'Historical data of measurement data of cumulative amount of reactive electric power consumption(lag) for power factor measurement'
		},
		0x90: { # General lighting class class
			0x80: 'Operation status',
			0xB0: 'Illuminance level',
			0xB1: 'Light color setting',
			0xB2: 'Illuminance level step setting',
			0xB3: 'Light color step setting',
			0xB4: 'Maximum specifiable values',
			0xB5: 'Maximum value of settable level for night lighting',
			0xB6: 'Lighting mode setting',
			0xB7: 'Illuminance level setting for main lighting',
			0xB8: 'Illuminance level step setting for main lighting',
			0xB9: 'Illuminance level setting for night lighting',
			0xBA: 'Illuminance level step setting for night lighting',
			0xBB: 'Light color setting for main lighting',
			0xBC: 'Light color level step setting for main lighting',
			0xBD: 'Light color setting for night lighting',
			0xBE: 'Light color level step setting for night lighting',
			0xBF: 'Lighting mode status in auto mode',
			0xC0: 'RGB setting for color lighting',
			0x90: 'ON timer reservation setting',
			0x91: 'ON timer setting',
			0x94: 'OFF timer reservation setting',
			0x95: 'OFF timer setting'
		},
		0x91: { # Single function lighting class
			0x80: 'Operation status',
			0xB0: 'Illuminance level setting'
		},
		0xA0: { # Buzzer class
			0x80: 'Operation status',
			0xB1: 'Sound generation setting',
			0xE0: 'Buzzer sound type'
		}
	},
	0x03: { # Cooking/housework-related device class group
		0xB2: { # Electric hot water pot (Electric thermos) class
			0x80: 'Operation status',
			0xB0: 'Cover open/close status',
			0xB1: 'No-water warning',
			0xB2: 'Boil-up setting',
			0xE0: 'Boil-up/warmer mode setting',
			0xE1: 'Set value of warmer temperature',
			0xE2: 'Hot water discharge status',
			0xE3: 'Lock status'
		},
		0xB7: { # Refrigerator class
			0x80: 'Operation status',
			0xB0: 'Door open/close status',
			0xB1: 'Door open warning',
			0xB2: 'Refrigerator compartment door status',
			0xB3: 'Freezer compartment door status',
			0xB4: 'Ice compartment door status',
			0xB5: 'Vegetable compartment door status',
			0xB6: 'Multi-refrigera-ting mode compartment door',
			0xE0: 'Maximum allowable temperature setting level',
			0xE2: 'Refrigerator compartment temperature setting',
			0xE3: 'Freezer compartment temperature setting',
			0xE4: 'Ice temperature setting',
			0xE5: 'Vegetable compartment temperature setting',
			0xE6: 'Multi-refrigera-ting mode compartment temperature setting',
			0xE9: 'Refrigerator compartment temperature level setting',
			0xEA: 'Freezer compartment temperature level setting',
			0xEB: 'ice compartment temperature level setting',
			0xEC: 'Vegetable compartment temperature level setting',
			0xED: 'Multi-refrigera-ting mode compartment temperature level setting',
			0xD1: 'Measured refrigerator compartment temperature',
			0xD2: 'Measured freezer compartment temperature',
			0xD3: 'Measured subzero-fresh compartment temperature',
			0xD4: 'Measured vegetable compartment temperature',
			0xD5: 'Measured multi-refrigeratin g mode compartment temperature',
			0xD8: 'Compressor rotation speed',
			0xDA: 'Measured electric current consumption',
			0xDC: 'Rated power consumption',
			0xA0: 'Quick freeze function setting',
			0xA1: 'Quick refrigeration function setting',
			0xA4: 'Icemaker setting',
			0xA5: 'Icemaker operation status',
			0xA6: 'Icemaker tank status',
			0xA8: 'Refrigerator compartment humidification function setting',
			0xA9: 'Vegetable compartment humidification function setting',
			0xAD: 'Deodorization function setting'
		},
		0xB8: { # Combination microwave oven (Electronic oven) class
			0x80: 'Operation status',
			0xB0: 'Door open/close status',
			0xB1: 'Heating status',
			0xB2: 'Heating setting',
			0xE0: 'Heating mode setting',
			0xE1: 'Automatic heating setting',
			0xE2: 'Automatic heating level setting',
			0xD0: 'Automatic heating menu setting',
			0xD1: 'Oven mode setting',
			0xD5: 'Oven preheating setting',
			0xD6: 'Fermenting mode setting',
			0xE3: 'Chamber temperature setting',
			0xE4: 'Food temperature setting',
			0xE5: 'Heating time setting',
			0xE6: 'Remaining heating time setting',
			0xE7: 'Microwave heating power setting',
			0xE8: 'Prompt message setting',
			0xE9: 'Accessories to combination microwave oven setting',
			0xEA: 'Display character string setting',
			0xEB: 'Two-stage microwave heating setting (duration)',
			0xEC: 'Two-stage microwave heating setting (heating power)'
		},
		0xB9: { # Cooking heater class
			0x80: 'Operation status',
			0xB1: 'Heating status',
			0xB2: 'Heating setting',
			0xB3: 'All stop setting',
			0xE7: 'Heating power setting',
			0xE3: 'Heating temperature setting',
			0xE0: '“Heating modes of stoves” setting',
			0x96: 'Relative time settings of OFF timers',
			0xA1: 'Child lock setting',
			0xA2: 'Radiant heater lock setting'
		},
		0xBB: { # Raice cooker class
			0x80: 'Operation status',
			0xB0: 'Cover open/close status',
			0xB1: 'Rice cooking status',
			0xB2: 'Rice cooking control setting',
			0xE1: 'Warmer setting',
			0xE5: 'Inner pot removal status',
			0xE6: 'Cover removal status',
			0x90: 'Rice cooking reservation setting',
			0x91: 'Set value of rice cooking reservation setting time',
			0x92: 'Set value of rice cooking reservation setting relative time'
		},
		0xC5: { # Washing machine class
			0x80: 'Operation status',
			0xB0: 'Door/cover open/close status',
			0xB2: 'Washing machine setting',
			0xE1: 'Current stage of washing cycle',
			0xE6: 'Time remaining to complete washing cycle',
			0x90: 'ON timer reservation setting',
			0x91: 'ON timer setting',
			0x92: 'Relative time-based ON timer setting'
		},
		0xCE: { # Commercial show case class
			0x80: 'Operation status',
			0xB0: 'Operating mode',
			0xBD: 'Discharge temperature measurement',
			0xE0: 'Internal lighting operation status',
			0xE1: 'External lighting operation status',
			0xE2: 'Compressor operation status',
			0xE3: 'Internal temperature measurement',
			0xE4: 'Freezing capability value',
			0xE5: 'Defrosting heater power consumption',
			0xE6: 'Fan motor power consumption',
			0xCA: 'Group information',
			0xE7: 'Heater mode'
		},
		0xC6: { # Clothes dryer class
			0x80: 'Operation status',
			0xB0: 'Door/cover open/close status',
			0xB2: 'Drying setting',
			0xE1: 'Drying status',
			0xE6: 'Remaining drying time',
			0x90: 'ON timer reservation setting',
			0x91: 'ON timer setting',
			0x92: 'Relative time-based ON timer setting'
		},
		0xD3: { # Washer and dryer class
			0x80: 'Operation status',
			0xB0: 'Door/cover open/close status',
			0xB2: 'Washer and dryer setting',
			0xD0: 'Washer and dryer cycle setting 1',
			0xD1: 'Washer and dryer cycle setting 2',
			0xD2: 'Drying cycle setting',
			0xD3: 'Washer and dryer cycle option list 1',
			0xD4: 'Washer and dryer cycle option list 2',
			0xD5: 'Washer and dryer cycle option list 3',
			0xD6: 'Water flow rate setting',
			0xD7: '“Rotation speed for spin drying” setting',
			0xD8: '“Degree of drying” setting',
			0xDB: 'Remaining washing time',
			0xDC: 'Remaining drying time',
			0xDF: 'Elapsed time on the ON timer',
			0xE1: 'Presoaking time setting',
			0xE2: 'Current stage of washer and dryer cycle',
			0xE3: 'Water volume setting 1',
			0xE4: 'Water volume setting 2',
			0xE5: 'Washing time setting',
			0xE6: '“Number of times of rinsing” setting',
			0xE7: 'Rinsing process setting',
			0xE8: 'Spin drying time setting',
			0xE9: 'Drying time setting',
			0xEA: 'Warm water setting',
			0xEB: 'Bathtub water recycle setting',
			0xEC: 'Wrinkling minimization setting',
			0xED: 'Time remaining to complete washer and dryer cycle',
			0xEE: 'Door/cover lock setting',
			0xEF: 'Washer and dryer cycle',
			0x90: 'ON timer reservation setting',
			0x91: 'ON timer setting',
			0x92: 'Relative time-based ON timer setting'
		},
		0xD4: { # Commercial show case outdoor unit class
			0x80: 'Operation status',
			0xAA: 'Exceptional status',
			0xB0: 'Operating mode',
			0xBE: 'Outdoor air temperature measurement',
			0xE2: 'Compressor operation status',
			0xCA: 'Group information'
		}
	},
	0x04: { # Health-related device class group
		0x01: { # Weighing machine class
			0x80: 'Operation status',
			0xE0: 'Measured value of body weight',
			0xE1: 'Measured value of body fat'
		}
	},
	0x05: { # Management/control-related device class group
		0xFD: { # Switch (supporting JEM-A/HA terminals) class
			0x80: 'Operation status',
			0xE0: 'Connected device'
		}
	},
	0x06: { # AV-related device class group
		0x01: { # Display class
			0x80: 'Operation status',
			0xB0: 'Display control setting',
			0xB1: 'Character string setting acceptance status',
			0xB2: 'Supported character codes',
			0xB3: 'Character string to present to the user',
			0xB4: 'Length of character string accepted '
		},
		0x02: { # Television class
			0x80: 'Operation status',
			0xB0: 'Display control setting',
			0xB1: 'Character string setting acceptance status',
			0xB2: 'Supported character codes',
			0xB3: 'Character string to present to the user',
			0xB4: 'Length of character string accepted'
		},
		0x03: { # Audio class
			0x80: 'Operation status',
			0xB8: 'Volume setting',
			0xB9: 'Mute setting',
			0xBC: 'Input source setting'
		},
		0x04: { # Network camera class
			0x80: 'Operation status',
			0xC0: 'Still image photography setting acceptance status',
			0xC1: 'Still image photography setting',
			0xD0: 'Transfer setting'
		}
	},
	0x0E: { # Profile class group
		0xF0: { # Node profile class
			0x80: ('Operating status'),
			0x82: ('Version information'),
			0x83: ('Identification number'),
			0x89: ('Fault content'),
			0xBF: ('Unique identifier data'),
			0xD3: ('Number of self-node instances'),
			0xD4: ('Number of self-node classes'),
			0xD5: ('Instance list notification'),
			0xD6: ('Self-node instance list S', f._0EF0D6),
			0xD7: ('Self-node class list S'),
			# Super Class of the Profile class group
			0x88: 'Fault status',
			0x8A: 'Manufacturer code',
			0x8B: 'Business facility code',
			0x8C: 'Product code',
			0x8D: 'Production number ',
			0x8E: 'Production date',
			0x9D: 'Status change announcement property map',
			0x9E: 'Set property map',
			0x9F: 'Get property map'
		}
	}
}



EPC_SUPER = {
	0x80: ('Operation status', None),
	0x81: ('Installation location', f._FF0081),
	0x82: ('Standard version information', f._FF0082),
	0x83: ('Identification number', None),
	0x84: ('Measured instantaneous power consumption', None),
	0x85: ('Measured cumulative power consumption', None),
	0x86: ('Manufacturer’s fault code', None),
	0x87: ('Current limit setting', None),
	0x88: ('Fault status', None),
	0x89: ('Fault description', None),
	0x8A: ('Manufacturer code', f._FF008A),
	0x8B: ('Business facility code', None),
	0x8C: ('Product code', None),
	0x8D: ('Production number', None),
	0x8E: ('Production date', None),
	0x8F: ('Power-saving operation setting', None),
	0x93: ('Remote control setting', None),
	0x97: ('Current time setting', None),
	0x98: ('Current date setting', None),
	0x99: ('Power limit setting', None),
	0x9A: ('Cumulative operating time', None),
	0x9B: ('SetM property map', None),
	0x9C: ('GetM property map', None),
	0x9D: ('Status change announcement property map', None),
	0x9E: ('Set property map', f._FF009E),
	0x9F: ('Get property map', f._FF009F)
}



from setuptools import setup

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(
    name='mitsubishi_echonet',
    version='0.2',
    author='Scott Phillips',
    author_email='jrh@example.com',
    packages=['mitsubishi_echonet'],
    url='http://pypi.python.org/pypi/mitsubishi_echonet/',
    license='LICENSE.txt',
    description='A library for interfacing with Mitsubishi HVAC via the Echonet lite protocol.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    # install_requires=[
    #    "Django >= 1.1.1",
    #    "caldav == 0.1.4",
    #],
)
